using System;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Messaging;



namespace IHSTI.MSMQMessaging
{

	public interface IMSMQ

	{
		bool IsEmpty();
		void SendMessage(object theMessage); //send any object as a message
		object ReadMessage(Type ObjectType); //collect a serialized object message
		
		//early development....
		StringCollection ReadMessage(); //collect a string collection message
	}

	public class MSMQ : IMSMQ
	{
		private System.Messaging.MessageQueue fQueue ;


		private void Attach_MSMQ(string QueuePath, string QueueLabel)
		{
			try
			{   
				//test if queue exists yet, create if not
				if (!System.Messaging.MessageQueue.Exists(QueuePath))
				{
					System.Messaging.MessageQueue.Create(QueuePath);
				}
				//create a messagequeue object over it
				fQueue = new System.Messaging.MessageQueue(QueuePath);
				//if it doesn't already have a label give it the new one
				if (fQueue.Label=="") 
					if (QueueLabel != "")
						fQueue.Label = QueueLabel;
			}
			catch (System.Messaging.MessageQueueException e)
			{
				// Handle Q not found error..
				if (e.MessageQueueErrorCode == 
					System.Messaging.MessageQueueErrorCode.QueueNotAvailable)
				{
					EventLog NTLog = new EventLog();
					// Write EXCEPTION MESSAGE to the event log.    
					NTLog.WriteEntry(e.Message);
				}
			}
		}
        
		public MSMQ(string QueuePath, string QueueLabel) //constructor
		{ 
			Attach_MSMQ(QueuePath, QueueLabel);
		}
		public MSMQ(string QueuePath) //constructor
		{ 
			Attach_MSMQ(QueuePath, "");
		}

		public bool IsEmpty()
		{
			bool theQueueIsEmpty = false;

			try
			{
				fQueue.Peek(new TimeSpan(0)); //is the queue non-empty?
				theQueueIsEmpty = false;

			}
			catch (System.Messaging.MessageQueueException e)
			{
				if (e.MessageQueueErrorCode == 
					System.Messaging.MessageQueueErrorCode.IOTimeout)
				{
					// No message was in the queue.
					theQueueIsEmpty = true;
				}
			}
			return theQueueIsEmpty;
		}



		public void SendMessage(object theMessage)
			{  
			fQueue.Send(theMessage);
		}


		public StringCollection ReadMessage()
		{
			//specify the XMLformatter for serialization, using a StringCollection type
			fQueue.Formatter = new System.Messaging.XmlMessageFormatter(new Type[] {typeof(StringCollection)});

			StringCollection theResult = new StringCollection();
			if (!IsEmpty()) //is the queue non-empty?
				if (fQueue.Peek() != null)
				{
					theResult = fQueue.Receive().Body as StringCollection;
				}
			return theResult;

		}

		public object ReadMessage(Type ObjectType)
		{
			fQueue.Formatter = new System.Messaging.XmlMessageFormatter(new Type[] {ObjectType});

			if (!IsEmpty()) //is the queue non-empty?
				if (fQueue.Peek() != null)
				{
					return fQueue.Receive().Body ;
				}

			return null;
		}
		#region IDisposable Members

		public void Dispose()
		{
			// TODO:  Add MSMQ.Dispose implementation
		}

		#endregion
	}
}




